# -*- coding:utf-8 -*-

import BaseHTTPServer
import json
import socket
import logging

class CNrpcServer:
    def __init__(self, net_interface):
        assert isinstance(net_interface, tuple)

        self._host, self.port = net_interface
        self._funcs = {}

    def register_function(self, func, name=None):
        assert callable(func)

        if name is None:
            name = func.func_name
        self._funcs[name] = func

    def serve_forever(self):
        s = socket.socket()
        s.bind((self._host, self.port))
        s.listen(5)

        while True:
            client, addr = s.accept()
            msg = client.recv(1024)
            request = json.loads(msg, encoding="utf-8")
            response = self._call_function(request["id"], request["method"], request["params"])
            str_response = json.dumps(response, encoding="utf-8")
            client.send(str_response)
            client.close()

    def _call_function(self, id, name, params):
        msg = dict(id=id)
        func = self._funcs.get(name)
        if func is None:
            msg["flag"] = False
            msg["result"] = "No function named '{}' has been registered".format(name)

        else:
            try:
                msg["flag"] = True
                msg["result"] = func(*params)
            except Exception as e:
                msg["flag"] = False
                msg["result"] = e.message

        print msg
        return msg


if __name__ == '__main__':
    server = CNrpcServer(("localhost", 7070))

    def add(a,b): return a+b

    server.register_function(add)

    server.serve_forever()


# -*- coding:utf-8 -*-
import socket
import json
from method import Method


class CNrpcClient:
    def __init__(self, host, port):
        self._host = host
        self._port = port

    def __getattr__(self, name):
        return Method(name, self._request)

    def _request(self, serialize_method):
        s = socket.socket()
        s.connect((self._host, self._port))
        s.send(json.dumps(serialize_method))
        res = s.recv(1024)
        s.close()

        res = json.loads(res, encoding="utf-8")

        assert res["id"] == serialize_method["id"]

        if res["flag"]:
            return res["result"]
        else:
            raise RuntimeError(res["result"])


if __name__ == '__main__':
    client = CNrpcClient("localhost", 7070)
    print client.add(1, 2)
    print client.add2(1, 2)




import uuid


class Method:
    def __init__(self, name, request):
        self._name = name
        self._request = request

    def __call__(self, *args):
        self._params = args
        return self._request(self._serialize())

    def _serialize(self):
        return {
            "id": uuid.uuid4().__str__(),
            "method": self._name,
            "params": self._params
        }



# -*- coding:utf-8 -*-
from os.path import dirname, join
from setuptools import find_packages
from distutils.core import setup

# with open(join(dirname(__file__), 'cnrpc/VERSION'), 'rb') as f:
#     version = f.read().decode('ascii').strip()
version = __import__("cnrpc").__version__

extras_require = {}

setup(
    name='cnrpc',
    version=version,
    url='http://moha.vip',
    description='Simple rpc module in computer network lesson',
    long_description=open('README.rst').read(),
    author='Tim Kong',
    maintainer='Tim Kong',
    maintainer_email='yjxkwp@foxmail.com',
    license='BSD',
    packages=find_packages(exclude=('tests', 'tests.*')),
    include_package_data=True,
    zip_safe=False,
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Environment :: Console',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Topic :: Internet :: WWW/HTTP',
        'Topic :: Software Development :: Libraries :: Application Frameworks',
        'Topic :: Software Development :: Libraries :: Python Modules',
    ],
    install_requires=[
    ],
    extras_require=extras_require,
)
